import json
import jsonschema

CONFIG_SCHEMA = {
    '$schema': 'http://json-schema.org/draft-07/schema#',

    'definitions': {
        'timeout': {
            'type': 'number',
            'maximum': 60,  # sanity
            'exclusiveMinimum': 0
        },
        'database-credentials': {
            'type': 'object',
            'properties': {
                'hostname': {'type': 'string'},
                'dbname': {'type': 'string'},
                'username': {'type': 'string'},
                'password': {'type': 'string'}
            },
            'required': ['hostname', 'dbname', 'username', 'password'],
            'additionalProperties': False
        },
        'ssh-credentials': {
            'type': 'object',
            'properties': {
                'username': {'type': 'string'},
                'private-key': {'type': 'string'},
                'known-hosts': {'type': 'string'}
            },
            'required': ['private-key', 'known-hosts'],
            'additionalProperties': False
        },
        'ims': {
            'type': 'object',
            'properties': {
                'api': {'type': 'string'},
                'username': {'type': 'string'},
                'password': {'type': 'string'}
            },
            'required': ['api', 'username', 'password'],
            'additionalProperties': False
        },
        'otrs-export': {
            'type': 'object',
            'properties': {
                'username': {'type': 'string'},
                'private-key': {'type': 'string'},
                'known-hosts': {'type': 'string'},
                'destination': {'type': 'string'}
            },
            'required': [
                'username',
                'private-key',
                'known-hosts',
                'destination'
            ],
            'additionalProperties': False
        },
        'redis-credentials': {
            'type': 'object',
            'properties': {
                'hostname': {'type': 'string'},
                'port': {'type': 'integer'},
                'celery-db-index': {'type': 'integer'},
                'socket_timeout': {'$ref': '#/definitions/timeout'}
            },
            'required': ['hostname', 'port'],
            'additionalProperties': False
        },
        'redis-sentinel-config': {
            'type': 'object',
            'properties': {
                'hostname': {'type': 'string'},
                'port': {'type': 'integer'},
                'celery-db-index': {'type': 'integer'},
                'name': {'type': 'string'},
                'redis_socket_timeout': {'$ref': '#/definitions/timeout'},
                'sentinel_socket_timeout': {'$ref': '#/definitions/timeout'}
            },
            'required': ['hostname', 'port', 'name'],
            'additionalProperties': False
        },
        'interface-address': {
            'type': 'object',
            'properties': {
                'address': {'type': 'string'},
                'network': {'type': 'string'},
                'interface': {'type': 'string'},
                'router': {'type': 'string'}
            },
            'required': ['address', 'network', 'interface', 'router'],
            'additionalProperties': False
        },
        'gws-direct-snmp-counter': {
            'type': 'object',
            'properties': {
                'hostname': {'type': 'string'},
                'community': {'type': 'string'},
                'oid': {
                    'type': 'string',
                    'pattern': r'^(\d+\.)*\d+$'
                }
            },
            'required': ['hostname', 'community', 'oid'],
            'additionalProperties': False
        },
        'gws-direct-snmp-counter-list': {
            'type': 'array',
            'items': {'$ref': '#/definitions/gws-direct-snmp-counter'},
            'minItems': 1
        },
        'gws-direct-counters': {
            'type': 'object',
            'properties': {
                'traffic_in': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'},
                'traffic_out': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'},
                'discards_in': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'},
                'discards_out': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'},
                'errors_in': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'},
                'errors_out': {
                    '$ref': '#/definitions/gws-direct-snmp-counter-list'}
            },
            'required': ['traffic_in', 'traffic_out'],
            'additionalProperties': False
        },
        'gws-direct-nren-isp-counters': {
            'type': 'object',
            'properties': {
                'nren': {'type': 'string'},
                'isp': {
                    'type': 'string',
                    'enum': ['Cogent', 'Telia', 'Century Link']
                },
                'counters': {'$ref': '#/definitions/gws-direct-counters'},
            },
            'required': ['nren', 'isp', 'counters'],
            'additionalProperties': False
        },
        'gws-direct': {
            'type': 'array',
            'items': {'$ref': '#/definitions/gws-direct-nren-isp-counters'}
        },
    },

    'type': 'object',
    'properties': {
        'ops-db': {'$ref': '#/definitions/database-credentials'},
        'ssh': {'$ref': '#/definitions/ssh-credentials'},
        'redis': {'$ref': '#/definitions/redis-credentials'},
        'sentinel': {'$ref': '#/definitions/redis-sentinel-config'},
        'ims': {'$ref': '#/definitions/ims'},
        'otrs-export': {'$ref': '#/definitions/otrs-export'},
        'redis-databases': {
            'type': 'array',
            'minItems': 1,
            'items': {'type': 'integer'}
        },
        'managed-routers': {'type': 'string'},
        'lab-routers': {
            'type': 'array',
            'items': {'type': 'string'}
        },
        'unmanaged-interfaces': {
            'type': 'array',
            'items': {'$ref': '#/definitions/interface-address'}
        },
        'gws-direct': {'$ref': '#/definitions/gws-direct'}

    },
    'oneOf': [
        {
            'required': [
                'ops-db',
                'ssh',
                'redis',
                'redis-databases',
                'otrs-export',
                'ims',
                'managed-routers',
                'gws-direct']
        },
        {
            'required': [
                'ops-db',
                'ssh',
                'sentinel',
                'redis-databases',
                'otrs-export',
                'ims',
                'managed-routers',
                'gws-direct']
        }
    ],
    'additionalProperties': False
}


def load(f):
    """
    Loads, validates and returns configuration parameters.

    Input is validated against this jsonschema:

    .. asjson:: inventory_provider.config.CONFIG_SCHEMA

    :param f: file-like object that produces the config file
    :return: a dict containing the parsed configuration parameters
    """
    config = json.loads(f.read())
    jsonschema.validate(config, CONFIG_SCHEMA)
    return config
